@@ -1,31 +1,31 @@
 package Class::PObject::Driver::file;
 
-# $Id: file.pm,v 1.11 2003/08/23 13:15:12 sherzodr Exp $
+# $Id: file.pm,v 1.15.2.2 2003/09/06 10:14:57 sherzodr Exp $
 
 use strict;
+#use diagnostics;
 use File::Spec;
 use Log::Agent;
-use base ('Class::PObject::Driver');
+use Class::PObject::Driver;
+use vars ('$f', '$VERSION', '@ISA');
 use Fcntl (':DEFAULT', ':flock', ':mode');
-use vars ('$f');
 
-# defining the format of the object file. Sample would be 'obj00129.cpo'
+@ISA = ("Class::PObject::Driver");
+$VERSION = '2.00';
 $f = 'obj%05d.cpo';
 
 # called when pobject's save() method is called. Note: this is not
 # the same as save() method as the one called by pobject. This is different!
 sub save {
-    my ($self, $object_name, $props, $columns) = @_;
+    my $self = shift;
+    my $class = ref($self) || $self;
+    my ($object_name, $props, $columns) = @_;
+
+    logtrc 3, "%s->save()", $class;
 
     # if 'id' does not already exist, we're being asked to save a newly
     # created object. Before we do that, we create a new id for the object:
-    if ( defined $columns->{id} ) {
-        logtrc 2, "'id' (%d) already exists. Updating the object data", $columns->{id}
-
-    } else {
-        $columns->{id} = $self->generate_id($object_name, $props) or return;
-        logtrc 2, "'id' was missing, created a new id (%d)", $columns->{id}
-    }
+    $columns->{id} ||= $self->generate_id($object_name, $props) or return;
 
     # _filename() returns the name of the file this particular object should
     # be stored in. Look into _filename() for details
@@ -45,7 +45,7 @@ sub save {
     }
 
     # and store frozen data into file:
-    print FH $self->freeze($columns);
+    print FH $self->freeze($object_name, $props, $columns);
     # if we can't close the file handle, it means we couldn't store it.
     unless( close(FH) ) {
         $self->errstr("couldn't save the object: $!");
@@ -53,25 +53,18 @@ sub save {
         return undef
     }
     # if everything went swell, we should return object id
-    logtrc 2, "save() successful. Object id is %d", $columns->{id};
     return $columns->{id}
 }
 
 
 
 
-sub load {
+sub load_ids {
     my $self = shift;
+    my $class = ref($self) || $self;
     my ($object_name, $props, $terms, $args) = @_;
 
-    # if we're asked to load by id, let's take a shortcut instead,
-    # thus more efficient
-    if ( $terms && (ref($terms) ne 'HASH') && ($terms =~m/^\d+$/) ) {
-        logtrc 2, "loading by 'id' (%d)", $terms;
-        return $self->load_by_id($object_name, $props, $terms) or return
-    }
-
-    logtrc 2, "loading object by 'terms' and/or 'args'";
+    logtrc 3, "%s->load_ids(@_)", $class;
 
     # if we come this far, we're being asked to return either all the objects,
     # or by some criteria
@@ -91,21 +84,16 @@ sub load {
         logerr $self->errstr;
         return undef
     }
-
-    logtrc 2, "Browsing '%s'", $object_dir;
-
     my $n = 0;
     while ( my ($filename, $stat) = each %files ) {
         # if 'limit' was given, and 'offset' is missing and sort is not given,
         # then check we have already reached our 'limit'. Otherwise, we need to
         # load all the objects to the memory before we can return the data set
         if ( defined($args->{limit}) && (!$args->{offset}) && (!$args->{'sort'}) && ($n == $args->{limit}) ) {
-            logtrc 2, "only 'limit' (%d) was defined and been met. Exiting the loop now", $args->{limit};
             last
         }
         # if it is a directory, then skip to the next file
         if ( S_ISDIR($stat->mode) ) {
-            logtrc 2, "'%s' is a directory. Skipping to the next", $filename;
             next
         }
 
@@ -116,7 +104,6 @@ sub load {
         $filef_pattern    =~ s/\./\\./g;
 
         unless ( $filename =~ m/^$filef_pattern$/ ) {
-            logtrc 2, "'%s' doesn't match the regex '%s'. Skipping", $filename, $filef_pattern;
             next
         }
         # we open the file with read-only flag
@@ -125,7 +112,6 @@ sub load {
             logerr $self->errstr;
             return undef
         }
-
         unless(flock(FH, LOCK_SH)) {
             $self->errstr("couldn't lock '$filename': $!");
             logerr $self->errstr;
@@ -136,17 +122,21 @@ sub load {
         unless( defined $datastr ) {
             next
         }
-        my $data = $self->thaw($datastr);
+        my $data = $self->thaw($object_name, $props, $datastr);
         if ( $self->_matches_terms($data, $terms) ) {
-            push @data_set, $data;
+            push @data_set, keys %$args ? $data : $data->{id};
             $n++
         }
     }
     untie(%files);
 
     # returning post-processed data set
-    logtrc 2, "%d objects were found matching the 'terms'", scalar(@data_set);
-    return $self->_filter_by_args(\@data_set, $args)
+    unless ( keys %$args ) {
+        return \@data_set
+    }
+
+    my $data_set = $self->_filter_by_args(\@data_set, $args);
+    return [ map { $_->{id} } @$data_set ]
 }
 
 
@@ -163,8 +153,12 @@ sub load {
 
 
 # load_by_id() is called only while object is to be retrieved by its id
-sub load_by_id {
-    my ($self, $object_name, $props, $id) = @_;
+sub load {
+    my $self = shift;
+    my $class = ref($self) || $self;
+    my ($object_name, $props, $id) = @_;
+
+    logtrc 3, "%s->load(%s)", $class, join ", ", @_;
 
     # determine the name of the file for this object
     my $filename = $self->_filename($object_name, $props, $id) or return;
@@ -188,7 +182,7 @@ sub load_by_id {
         $self->errstr("object is empty");
         return undef
     }
-    return [$self->thaw($data_str)]
+    return $self->thaw($object_name, $props, $data_str)
 }
 
 
@@ -281,33 +275,25 @@ sub _dir {
             return undef
         }
     }
-
-  # return the directory
-  return $object_dir
+    # return the directory
+    return $object_dir
 }
 
-
-
-
-
-
-
 1;
 
 __END__;
 
-=pod
-
 =head1 NAME
 
 Class::PObject::Driver::file - Default PObject driver
 
 =head1 SYNOPSIS
 
-  pobject Person => {
-    columns   => ['id', 'name', 'email']
-    datasource=> 'data'
-  };
+    pobject Person => {
+        columns   => ['id', 'name', 'email']
+        datasource=> 'data',
+        serializer => 'xml'
+    };
 
 =head1 DESCRIPTION
 
@@ -323,6 +309,11 @@ This data source is a folder in your operating system, inside which objects will
 Pobject will create a folder for each object type inside the I<datasource> folder, and will store
 all the objects of the same type in their own folders.
 
+Other supported property is I<serialiazer>, which defaults to I<storable> if the value is missing.
+This serializer defines the serializing and de-serializing method used by object driver.
+Possible values are I<xml>, which requires L<XML::Dumper|XML::Dumper> to have been installed,
+and I<dumper>, which requires L<Data::Dumper|Data::Dumper> and 'storable'.
+
 =head1 SUPPORTED FEATURES
 
 Class::PObject::Driver::file overrides following methods of Class::PObject::Driver
@@ -377,50 +368,20 @@ C<_filename($self, $pobject_name, \%properties)> is called to get a path to a fi
 object should be stored into. C<_filename()> will call C<_dir()> method to get the object directory,
 and builds a filename inside this directory.
 
-=item *
-
-C<freeze($self, \%data)> and C<thaw($self, $string)> are used to serialize and de-serialize the data
-to make it suitable for storing into disk. By default C<freeze()> and C<thaw()> use
-L<Data::Dumper|Data::Dumper>. If you want to use some other method, you can subclass
-Class::PObject::Driver::file and define your own C<freeze()> and C<thaw()> methods:
-
-    # Inside Class/PObject/Driver/my_file.pm
-    package Class::PObject::Driver::my_file;
-    use base ('Class::PObject::Driver::file');
-    require Storable;
-
-    sub freeze {
-        my ($self, $data) = @_;
-        return Storable::freeze($data)
-    }
-
-    sub thaw {
-        my ($self, $string) = @_;
-        return Storable::thaw($string)
-    }
-
-    1;
-
-    # Inside Article.pm, for example:
-    package Article;
-    pobject {
-        columns => ['id', 'title', 'author', 'content'],
-        driver  => 'my_file'
-    };
-
 =back
 
 =head1 OBJECT STORAGE
 
 Each object is stored as a separate file. File name pattern for each object file is defined in
-C<$Class::PObject::Driver::file::f> global variable, is is C<obj%05.cpo> by default, where C<%05>
-will be replaced with the I<id> of the object, zeroes padded if necessary.
+C<$Class::PObject::Driver::file::f> global variable, and is C<obj%05.cpo> by default, where C<%05>
+will be replaced with the I<id> of the object, zero-padded if necessary.
 
 B<Note:> extension '.cpo' stands for B<C>lass::B<PO>bject.
 
 =head1 SERIALIZATION
 
-Objects are serialized using standard L<Data::Dumper|Data::Dumper>
+Objects are serialized and de-serialized with the help of C<freeze()> and C<thaw()> methods
+provided by its base class, L<Class::PObject::Driver|Class::PObject::Driver>.
 
 =head1 ID GENERATION
 
@@ -432,24 +393,25 @@ the path to this folder) in a file called "counter.cpo".
 
 Removing F<counter.cpo> from the directory will force PObject to reset object ids. This may be a problem
 if there already are objects in the directory, and they may be overridden by new ids. I realize
-this is a scary limitation, which will be eventually addressed.
+this is a scary limitation, which will eventually be addressed.
 
-In the meanwhile, just don't make habit of removing F<counter.cpo>!
+In the meanwhile, just don't make habit of removing F<counter.cpo> :-).
 
 =head1 EFFICIENCY
 
 Since the driver doesn't keep an index of any kind, the most efficient way of loading the data is by its id.
-A relatively simple C<load(undef, {limit=>n})> syntax is also relatively fast.
+A relatively simple C<load(undef, {limit=>n})> syntax is also reasonably efficient.
 
-  my $p       = Person->load(451);
-  my @people  = Person->load();
-  my @group   = Person->load(undef, {limit=>100});
+    $p       = Person->load(451);
+    @people  = Person->load();
+    @group   = Person->load(undef, {limit=>100});
 
 as load() becomes complex, the performance gets degrading:
 
-  my @people = Person->load({name=>"Sherzod"}, {sort=>'age', direction=>'desc', limit=>10, offset=>4});
+    @people = Person->load( {name=>"Sherzod"}, 
+                            {sort=>'age', direction=>'desc', limit=>10, offset=>4} );
 
-To perform the above search, the driver walks through all the objects available in the I<datasource>, pushes all the objects matching 'name="sherzod"' to the data-set, then, just before returning the data set, performs sort, limit and offset calculations. 
+To perform the above search, the driver walks through all the objects available in the I<datasource>, pushes all the objects matching 'name="sherzod"' to the data-set, then, just before returning the data set, performs sort, limit and offset calculations.
 
 As you imagine, as the number of objects in the datasource increases, this operation will become more costly.
 
@@ -458,8 +420,8 @@ As you imagine, as the number of objects in the datasource increases, this opera
 L<Class::PObject>, L<Class::PObject::Driver::mysql>,
 L<Class::PObject::Driver::file>
 
-=head1 AUTHOR
+=head1 COPYRIGHT AND LICENSE
 
-Sherzod Ruzmetov <sherzodr@cpan.org>
+For author and copyright information refer to Class::PObject's L<online manual|Class::PObject>.
 
 =cut
