@@ -1,13 +1,15 @@
 package Class::PObject::Driver::mysql;
 
-# $Id: mysql.pm,v 1.16 2003/08/23 10:36:44 sherzodr Exp $
+# $Id: mysql.pm,v 1.19.4.2 2003/09/06 10:14:58 sherzodr Exp $
 
 use strict;
+#use diagnostics;
 use Log::Agent;
-use vars qw(@ISA $VERSION);
+use vars ('@ISA', '$VERSION');
 require Class::PObject::Driver::DBI;
 
 @ISA = ('Class::PObject::Driver::DBI');
+$VERSION = '2.00';
 
 # 
 # overriding _prepare_insert() with the version that uses REPLACE
@@ -22,7 +24,6 @@ sub _prepare_insert {
         push @fields, "$k=?";
         push @bind_params, $v
     }
-
     $sql .= join(", ", @fields);
     return ($sql, \@bind_params)
 }
@@ -37,13 +38,15 @@ sub save {
     my $dbh                 = $self->dbh($object_name, $props)        or return;
     my $table               = $self->_tablename($object_name, $props) or return;
     my ($sql, $bind_params) = $self->_prepare_insert($table, $columns);
-    my $sth                 = $dbh->prepare( $sql );
 
+    $self->_write_lock($dbh, $table) or return undef;
+    my $sth                 = $dbh->prepare( $sql );
     unless ( $sth->execute(@$bind_params) ) {
         $self->errstr("couldn't save/update the record: " . $sth->errstr);
         logerr $self->errstr;
         return undef
     }
+    $self->_unlock($dbh) or return undef;
     return $dbh->{mysql_insertid} || $dbh->{insertid}
 }
 
@@ -64,50 +67,39 @@ sub dbh {
         $self->errstr("'datasource' is invalid");
         return undef
     }
-
     if ( defined $props->{datasource}->{Handle} ) {
         return $props->{datasource}->{Handle}
     }
-  
     my $stashed_name = $props->{datasource}->{DSN};
     if ( defined $self->stash($stashed_name) ) {
         return $self->stash($stashed_name)
     }
-  
     if ( $datasource->{Handle} ) {
         $self->stash($stashed_name, $datasource->{Handle});
         return $datasource->{Handle}
     }
-  
     my $dsn       = $datasource->{DSN};
     my $db_user   = $datasource->{User} || $datasource->{UserName};
     my $db_pass   = $datasource->{Password};
-
     unless ( $dsn ) {
         $self->errstr("'DSN' is missing in 'datasource'");
         return undef
     }
-  
     require DBI;
     my $dbh = DBI->connect($dsn, $db_user, $db_pass, {RaiseError=>1, PrintError=>0});
     unless ( defined $dbh ) {
         $self->errstr("couldn't connect to 'DSN': " . $DBI::errstr);
         return undef
     }
+    $dbh->{FetchHashKeyName} = 'NAME_lc';
     $self->stash($stashed_name, $dbh);
     $self->stash('close', 1);
     return $dbh
 }
 
-
-
-
-
 1;
 __END__;
 
-=pod
-
 =head1 NAME
 
 Class::PObject::Driver::mysql - MySQL Pobject Driver
@@ -196,15 +188,8 @@ that MySQL will take care of creating auto-incrementing unique object ids for yo
 L<Class::PObject>, L<Class::PObject::Driver::csv>,
 L<Class::PObject::Driver::file>
 
-=head1 AUTHOR
-
-Sherzod Ruzmetov E<lt>sherzodr@cpan.orgE<gt>
-
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003 by Sherzod B. Ruzmetov.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself. 
+For author and copyright information refer to Class::PObject's L<online manual|Class::PObject>.
 
 =cut
